defmodule ExRedi.Redis do
  @moduledoc """
  ExRedi Redis
  """
  use GenServer

  # ====== #
  # Client #
  # ====== #

  @spec start_link(keyword) :: GenServer.on_start()
  def start_link(opts \\ []) do
    GenServer.start_link(__MODULE__, opts, name: __MODULE__)
  end

  def command(args) do
    GenServer.call(__MODULE__, {:command, args})
  end

  # ====== #
  # Server #
  # ====== #

  def init(opts) do
    Process.flag(:trap_exit, true)

    with {behaviour, connection} <- split_opts(opts),
         {:ok, name} <- Redix.start_link(behaviour, connection) do
      {:ok, %{name: name, behaviour: behaviour, connection: connection}}
    end
  end

  def handle_call({:command, args}, _, %{name: name} = state) do
    {:reply, redix_command(name, args), state}
  end

  # ======= #
  # Private #
  # ======= #

  @spec split_opts(keyword) :: {{keyword, keyword}, keyword}
  defp split_opts(opts) do
    {behaviour, _} = Keyword.pop(opts, :redix_behaviour, [])
    {connection, _} = Keyword.pop(opts, :redix_connection, [])

    {
      merge_opts(:redix_behaviour, behaviour),
      merge_opts(:redix_connection, connection)
    }
  end

  @spec merge_opts(atom, keyword) :: keyword
  defp merge_opts(key, opts) do
    :ex_redi
    |> Application.get_env(key, [])
    |> Keyword.merge(opts)
  end

  defp redix_command(name, args) do
    Redix.command(name, List.flatten(args))
  rescue
    error in Redix.Error ->
      {:error, Exception.message(error)}
  end
end
